@@ -1,6 +1,9 @@
 package Apache::Debug;
-
+use Cwd 'fastcwd';
 #from HTTP::Status
+use vars qw($VERSION);
+$VERSION = (qw$Revision: 1.16 $)[1];
+
 my %StatusCode = (
     100 => 'Continue',
     101 => 'Switching Protocols',
@@ -47,30 +50,30 @@ sub dump {
     my $conn = $r->connection;
     my %headers = $r->headers_in;
     my $host = $r->get_remote_host;
-
+    my $cwd = fastcwd;
     $r->status($status);
     $r->content_type("text/html");
     $r->content_language("en");
     $r->no_cache(1);
-    $r->header_out("X-Debug-Version" => q$Id: Debug.pm,v 1.13 1996/12/19 04:14:51 dougm Exp $);
+    $r->header_out("X-Debug-Version" => q$Id: Debug.pm,v 1.16 1997/06/03 03:00:42 dougm Exp $);
     $r->send_http_header;
     
-    return 0 if $r->method eq "HEAD";   # should not generate a body
+    return 0 if $r->header_only;   # should not generate a body
 
     my $title = "$status $StatusCode{$status}";
     $r->write_client(join("\n", "<html>",
 			  "<head><title>$title</title></head>",
                           "<body>", "<h3>$title</h3>", @_, 
-			  "<pre>", ($@ ? "$@\n" : "")));
+			  "<pre>", ($@ ? "$@\n" : ""), "cwd=$cwd\n"));
 
     for (
 	 qw(
 	    method uri protocol path_info filename
-            allow_options is_perlaliased
+            allow_options
 	    )
 	 )
     {
-	$r->write_client(sprintf "<b>\$r->%-17s</b> : %s\n", $_, $r->$_() );
+	$r->print(sprintf "<b>\$r->%-17s</b> : %s\n", $_, $r->$_() );
     }
 
     for (
@@ -81,7 +84,7 @@ sub dump {
 	    )
 	 ) 
     {
-	$r->write_client(sprintf "<b>\$s->%-17s</b> : %s\n", $_, $srv->$_() );
+	$r->print(sprintf "<b>\$s->%-17s</b> : %s\n", $_, $srv->$_() );
     }
 
     for (
@@ -94,13 +97,13 @@ sub dump {
 	    )
 	 )
     {
-	$r->write_client(sprintf "<b>\$c->%-17s</b> : %s\n", $_, $conn->$_() );
+	$r->print(sprintf "<b>\$c->%-17s</b> : %s\n", $_, $conn->$_() );
     }
 
     my $args = $r->args;
     my %args = $r->args;
     my %in   = $r->content;
-    $r->write_client(
+    $r->print(
 		     "\n<b>scalar \$r->args       :</b> $args\n",
 			 
 		     "\n<b>\$r->args:</b>\n",
@@ -112,7 +115,7 @@ sub dump {
 		     "\n<b>\$r->headers_in:</b>\n",		     
 		     (map { sprintf "   %-12s = %s\n", $_, $headers{$_} } sort keys %headers),
 		     );
-    $r->write_client("</pre>\n</body></html>\n");
+    $r->print("</pre>\n</body></html>\n");
     return 0; #need to give a return status
 }
 
