@@ -1,18 +1,22 @@
 package Apache::Registry;
 use Apache ();
 #use strict; #eval'd scripts will inherit hints
-use Apache::Debug ();
 use Apache::Constants qw(:common &OPT_EXECCGI);
 use FileHandle ();
+use File::Basename qw(dirname);
+use Cwd qw(fastcwd);
 
 use vars qw($VERSION $Debug);
-#$Id: Registry.pm,v 1.27 1997/03/20 23:15:20 dougm Exp $
-$VERSION = (qw$Revision: 1.27 $)[1];
+#$Id: Registry.pm,v 1.32 1997/06/12 00:17:41 dougm Exp $
+$VERSION = (qw$Revision: 1.32 $)[1];
 
 $Debug ||= 0;
 # 1 => log recompile in errorlog
 # 2 => Apache::Debug::dump in case of $@
 # 4 => trace pedantically
+Apache->module('Apache::Debug') if $Debug;
+
+my(@cleanup);
 
 sub handler {
     my($r) = @_;
@@ -88,7 +92,9 @@ sub handler {
 			    '',
 			    'package ',
 			    $package,
-			    ';use Apache qw(exit warn);sub handler {',
+ 			    ';use Apache qw(exit);',
+ 			    'sub handler {',
+ 			    "\n#line 1 $filename\n",
 			    $sub,
 			    "\n}", # last line comment without newline?
 			   );
@@ -108,14 +114,29 @@ sub handler {
 	    $Apache::Registry->{$package}{mtime} = $mtime;
 	}
 
+	my $cwd = fastcwd;
+	chdir dirname $r->filename;
 	eval {$package->handler;};
+	chdir $cwd;
 	$^W = $oldwarn;
-	if ($@) {
-	    $r->log_error($@);
+
+	my $err = $@;
+	if($INC{'CGI.pm'}) {
+	    use Exporter ();
+	    eval { Exporter::require_version('CGI', 2.35); }; 
+	    $err .= $@ if $@;
+	}
+	if ($err) {
+	    $r->log_error($err);
 	    return SERVER_ERROR unless $Debug & 2;
 	    return Apache::Debug::dump($r, SERVER_ERROR);
 	}
-	return $r->status;
+
+	#run cleanup handlers if any
+	for (@cleanup) { &{$_}($r) }
+	@cleanup = ();
+
+	return OK;
     } else {
 	return NOT_FOUND unless $Debug & 2;
 	return Apache::Debug::dump($r, NOT_FOUND);
@@ -152,6 +173,14 @@ sub parse_cmdline {
     return $sub;
 }
 
+sub push_cleanup {
+    my $self = shift;
+    push @cleanup, @_;
+}
+
+#trick so we show up under CPAN/modules/by-module/CGI/
+package CGI::Apache;
+
 1;
 
 __END__
@@ -220,8 +249,7 @@ before running the script.
 
 =head1 ENVIRONMENT
 
-The Apache functions `warn' and `exit' override the Perl core built-in
-functions.
+The Apache function `exit' overrides the Perl core built-in function.
 
 The environment variable B<GATEWAY_INTERFACE> is set to C<CGI-Perl/1.1>.
 
